import json

class Builder:

    def __init__(self, asm_filename, csm_filename):
        self.asm_filename = asm_filename
        self.csm_filename = csm_filename

        self.old_asm = None
        self.old_csm = None

        self.asm_nodes = {}
        self.asm_edges = []

        self.csm_nodes = {}
        self.csm_edges = []

        self.node_mapping = {}

    def parse_file(self, filename):

        with open(filename) as f:
            lines = [json.loads(line) for line in f]

        return lines[0]

    def parse_asm(self):
        self.old_asm = self.parse_file(self.asm_filename)

        next_num = 0

        for i, node in enumerate(self.old_asm):


            node_type : str = node['type']
            if node_type.lower() == "class":
                new_node = {
                    "name": {
                        "type": "string",
                        "value": node["name"]
                    },
                    "attributes": {
                        "type": "list<$ATTRIBUTE>",
                        "value": []
                    },
                    "constraints": {
                        "type": "list<$EVENT_HANDLER>",
                        "value": []
                    },
                    "actions": {
                        "type": "list<$EVENT_HANDLER>",
                        "value": []
                    },
                    "cardinalities": {
                        "type": "list<$CARDINALITY>",
                        "value": []
                    },
                    "abstract": {
                        "type": "boolean",
                        "value": False
                    },
                    "$type": "/Formalisms/__LanguageSyntax__/SimpleClassDiagram/SimpleClassDiagram/Class"
                }


                self.node_mapping[node["name"]] = next_num

                self.asm_nodes[str(next_num)] = new_node
                next_num += 1

    def parse_csm(self):
        self.old_csm = self.parse_file(self.csm_filename)

        next_num = 0
        for i, node in enumerate(self.old_csm):


            node_type: str = node['type']

            if node_type.lower() == "text":
                node_name = node["text"].split()[0]


                if node_name not in self.node_mapping:
                    continue

                new_node = {
                    "typename": {
                        "type": "string",
                        "value": "ClassIcon"
                    },
                    "position": {
                        "type": "list<double>",
                        "value": [
                            605,
                            430
                        ]
                    },
                    "orientation": {
                        "type": "double",
                        "value": 0
                    },
                    "scale": {
                        "type": "list<double>",
                        "value": [
                            1,
                            1
                        ]
                    },
                    "mapper": {
                        "type": "code",
                        "value": ""
                    },
                    "parser": {
                        "type": "code",
                        "value": ""
                    },
                    "$contents": {
                        "type": "map<string,*>",
                        "value": {
                            "nodes": {
                                "5": {
                                    "width": {
                                        "type": "double",
                                        "value": 200
                                    },
                                    "height": {
                                        "type": "double",
                                        "value": 125
                                    },
                                    "cornerRadius": {
                                        "type": "double",
                                        "value": 5
                                    },
                                    "style": {
                                        "type": "map<string,string>",
                                        "value": {
                                            "stroke": "#000000",
                                            "stroke-dasharray": "",
                                            "fill": "#ffffff",
                                            "fill-opacity": 0.75,
                                            "stroke-width": 2
                                        }
                                    },
                                    "mapper": {
                                        "type": "code",
                                        "value": ""
                                    },
                                    "parser": {
                                        "type": "code",
                                        "value": ""
                                    },
                                    "$type": "/Formalisms/__LanguageSyntax__/ConcreteSyntax/ConcreteSyntax/Rectangle",
                                    "position": {
                                        "type": "list<double>",
                                        "value": [
                                            0,
                                            0
                                        ]
                                    },
                                    "orientation": {
                                        "type": "double",
                                        "value": 0
                                    },
                                    "scale": {
                                        "type": "list<double>",
                                        "value": [
                                            1,
                                            1
                                        ]
                                    }
                                },
                                "6": {
                                    "textContent": {
                                        "type": "string",
                                        "value": node_name
                                    },
                                    "style": {
                                        "type": "map<string,string>",
                                        "value": {
                                            "stroke": "#000000",
                                            "fill": "#ffffff",
                                            "fill-opacity": 0.75,
                                            "font-size": "20px",
                                            "font-style": "normal"
                                        }
                                    },
                                    "mapper": {
                                        "type": "code",
                                        "value": "var style = \n    {\n       \"stroke\": \"#000000\",\n       \"fill\": \"#ffffff\",\n       \"fill-opacity\": 0.75,\n       \"font-size\": \"20px\",\n       \"font-style\": (getAttr('abstract') ? 'italic' : 'normal')\n    };\n\n({'textContent': getAttr('name'), 'style':style})\n\n\n\n"
                                    },
                                    "parser": {
                                        "type": "code",
                                        "value": "({'name': getAttr('textContent')})"
                                    },
                                    "$type": "/Formalisms/__LanguageSyntax__/ConcreteSyntax/ConcreteSyntax/Text",
                                    "position": {
                                        "type": "list<double>",
                                        "value": [
                                            23,
                                            2
                                        ]
                                    },
                                    "orientation": {
                                        "type": "double",
                                        "value": 0
                                    },
                                    "scale": {
                                        "type": "list<double>",
                                        "value": [
                                            1,
                                            1
                                        ]
                                    }
                                },
                                "7": {
                                    "$type": "/Formalisms/__LanguageSyntax__/ConcreteSyntax/ConcreteSyntax/Contain",
                                    "position": {
                                        "type": "list<double>",
                                        "value": [
                                            13.896110991026944,
                                            3.0451514439171206
                                        ]
                                    },
                                    "orientation": {
                                        "type": "double",
                                        "value": 0
                                    },
                                    "scale": {
                                        "type": "list<double>",
                                        "value": [
                                            1,
                                            1
                                        ]
                                    },
                                    "link-style": {
                                        "type": "map<string,string>",
                                        "value": {
                                            "stroke": "#00ffff",
                                            "stroke-dasharray": "",
                                            "stroke-opacity": 0.1,
                                            "arrow-start": "none",
                                            "arrow-end": "classic-wide-long"
                                        }
                                    }
                                },
                                "17": {
                                    "segments": {
                                        "type": "string",
                                        "value": "m0,0l200,0"
                                    },
                                    "style": {
                                        "type": "map<string,string>",
                                        "value": {
                                            "stroke": "#000000",
                                            "stroke-dasharray": "",
                                            "fill": "#ffffff",
                                            "fill-opacity": 0.75,
                                            "font-size": "20px",
                                            "stroke-width": 2,
                                            "arrow-start": "none",
                                            "arrow-end": "none"
                                        }
                                    },
                                    "mapper": {
                                        "type": "code",
                                        "value": ""
                                    },
                                    "parser": {
                                        "type": "code",
                                        "value": ""
                                    },
                                    "$type": "/Formalisms/__LanguageSyntax__/ConcreteSyntax/ConcreteSyntax/Path",
                                    "position": {
                                        "type": "list<double>",
                                        "value": [
                                            1,
                                            26
                                        ]
                                    },
                                    "orientation": {
                                        "type": "double",
                                        "value": 0
                                    },
                                    "scale": {
                                        "type": "list<double>",
                                        "value": [
                                            1,
                                            1
                                        ]
                                    }
                                },
                                "18": {
                                    "textContent": {
                                        "type": "string",
                                        "value": ""
                                    },
                                    "style": {
                                        "type": "map<string,string>",
                                        "value": {
                                            "stroke": "#000000",
                                            "stroke-dasharray": "",
                                            "fill": "#ffffff",
                                            "fill-opacity": 0.75,
                                            "font-size": "13px",
                                            "stroke-width": 1,
                                            "arrow-start": "none",
                                            "arrow-end": "none"
                                        }
                                    },
                                    "mapper": {
                                        "type": "code",
                                        "value": "var filenames = getAttr('attributes').map(\n\t\t\t\tfunction(attr)\n\t\t\t\t{\n\t\t\t\t\treturn attr['name'];\n\t\t\t\t});\n({'textContent': filenames.join('\\n')})"
                                    },
                                    "parser": {
                                        "type": "code",
                                        "value": ""
                                    },
                                    "$type": "/Formalisms/__LanguageSyntax__/ConcreteSyntax/ConcreteSyntax/Text",
                                    "position": {
                                        "type": "list<double>",
                                        "value": [
                                            3,
                                            25
                                        ]
                                    },
                                    "orientation": {
                                        "type": "double",
                                        "value": 0
                                    },
                                    "scale": {
                                        "type": "list<double>",
                                        "value": [
                                            1,
                                            1
                                        ]
                                    }
                                },
                                "19": {
                                    "width": {
                                        "type": "boolean",
                                        "value": True
                                    },
                                    "ratio": {
                                        "type": "double",
                                        "value": 1
                                    },
                                    "$type": "/Formalisms/__LanguageSyntax__/ConcreteSyntax/ConcreteSyntax/Span",
                                    "position": {
                                        "type": "list<double>",
                                        "value": [
                                            168.09129891774523,
                                            30.625911571363645
                                        ]
                                    },
                                    "orientation": {
                                        "type": "double",
                                        "value": 0
                                    },
                                    "scale": {
                                        "type": "list<double>",
                                        "value": [
                                            1,
                                            1
                                        ]
                                    },
                                    "link-style": {
                                        "type": "map<string,string>",
                                        "value": {
                                            "stroke": "#0000ff",
                                            "stroke-dasharray": "",
                                            "stroke-opacity": 1,
                                            "arrow-start": "none",
                                            "arrow-end": "classic-wide-long"
                                        }
                                    }
                                },
                                "20": {
                                    "$type": "/Formalisms/__LanguageSyntax__/ConcreteSyntax/ConcreteSyntax/Contain",
                                    "position": {
                                        "type": "list<double>",
                                        "value": [
                                            6.49899850809561,
                                            18.998998508095553
                                        ]
                                    },
                                    "orientation": {
                                        "type": "double",
                                        "value": 0
                                    },
                                    "scale": {
                                        "type": "list<double>",
                                        "value": [
                                            1,
                                            1
                                        ]
                                    },
                                    "link-style": {
                                        "type": "map<string,string>",
                                        "value": {
                                            "stroke": "#00ffff",
                                            "stroke-dasharray": "",
                                            "stroke-opacity": 0.1,
                                            "arrow-start": "none",
                                            "arrow-end": "classic-wide-long"
                                        }
                                    }
                                }
                            },
                            "edges": [
                                {
                                    "src": "5",
                                    "dest": "7"
                                },
                                {
                                    "src": "7",
                                    "dest": "6"
                                },
                                {
                                    "src": "17",
                                    "dest": "19"
                                },
                                {
                                    "src": "19",
                                    "dest": "5"
                                },
                                {
                                    "src": "5",
                                    "dest": "20"
                                },
                                {
                                    "src": "20",
                                    "dest": "18"
                                }
                            ]
                        }
                    },
                    "$asuri": {
                        "type": "string",
                        "value": "/Formalisms/__LanguageSyntax__/SimpleClassDiagram/SimpleClassDiagram/Class/{}.instance".format(self.node_mapping[node_name])
                    },
                    "$type": "/Formalisms/__LanguageSyntax__/SimpleClassDiagram/SimpleClassDiagram.defaultIcons/ClassIcon"
                }

                self.csm_nodes[str(next_num)] = new_node
                next_num += 1

    def create_model(self, out_filename):
        out_asm = {
            "nodes": self.asm_nodes,
            "edges": self.asm_edges,
            "metamodels": ["/Formalisms/__LanguageSyntax__/SimpleClassDiagram/SimpleClassDiagram"]
        }
        out_csm = {
            "nodes": self.csm_nodes,
            "edges": self.csm_edges,
            "metamodels": ["/Formalisms/__LanguageSyntax__/SimpleClassDiagram/SimpleClassDiagram.defaultIcons"]
        }

        out_model = {"csm" : out_csm, "asm" : out_asm}

        with open(out_filename, 'w') as f:
            f.write(json.dumps(out_model, indent=4, sort_keys=False))



if __name__ == "__main__":
    name = "families"

    asm_filename = name + ".json"
    csm_filename = name + '.render.json'

    out_filename = name + ".model"

    b = Builder(asm_filename, csm_filename)
    b.parse_asm()
    b.parse_csm()
    b.create_model(out_filename)